@@ -9,7 +9,7 @@ This module generates the dual metadata system used by both ZIP and FOLDER conta
    - Includes internal:parent_id for relational queries (all levels)
    - Includes internal:relative_path for fast SQL queries (level 1+ only)
    - Used for: sql queries, navigation, statistics
-   
+
 2. LOCAL METADATA (DATA/folder/__meta__ files):
    - One file per FOLDER (only for level 1+)
    - Contains only the direct children of that specific folder
@@ -26,7 +26,7 @@ CRITICAL DESIGN PRINCIPLES:
   * Padding (__TACOPAD__*) ensures structural uniformity
 """
 
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, cast
 
 import polars as pl
 
@@ -35,21 +35,15 @@ from tacotoolbox._column_utils import (
     remove_empty_columns,
     reorder_internal_columns,
 )
-from tacotoolbox._constants import (
-    METADATA_PARENT_ID,
-    METADATA_RELATIVE_PATH,
-    is_padding_id,
-)
+from tacotoolbox._constants import METADATA_PARENT_ID, METADATA_RELATIVE_PATH
+from tacotoolbox._utils import is_padding_id
+from tacotoolbox.tortilla.datamodel import Tortilla
 
 if TYPE_CHECKING:
     from tacotoolbox.sample.datamodel import Sample
     from tacotoolbox.taco.datamodel import Taco
 
 
-class PITValidationError(Exception):
-    """Raised when Position-Isomorphic Tree (PIT) constraint is violated."""
-
-
 class MetadataPackage:
     """
     Complete metadata bundle for TACO containers.
@@ -124,18 +118,12 @@ class MetadataGenerator:
 
             dataframes.append(df)
 
-            # Validate PIT constraints
-            if depth == 0:
-                self._validate_pit_level0(df)
-            else:
-                self._validate_pit_depth(df, dataframes[depth - 1], depth)
-
         # Add internal:relative_path for fast SQL queries
         dataframes = self._add_relative_paths(dataframes)
 
         # Generate schemas
         pit_schema = generate_pit_schema(dataframes, debug=self.debug)
-        field_schema = generate_field_schema(dataframes)
+        field_schema = generate_field_schema(dataframes, self.taco)
 
         levels = dataframes
 
@@ -220,18 +208,20 @@ class MetadataGenerator:
     def _generate_folder_metadata(self, folder_sample: "Sample") -> pl.DataFrame:
         """
         Generate local metadata for a single folder.
-        
+
         Critical: Children may have different extensions (e.g., cloudmask vs s2data vs thumbnail),
         resulting in different schema widths. Must align schemas before concatenation.
         """
-        samples = folder_sample.path.samples
+        # Cast path to Tortilla to access .samples
+        tortilla = cast(Tortilla, folder_sample.path)
+        samples = tortilla.samples
         metadata_dfs = [s.export_metadata() for s in samples]
-        
+
         # Align schemas before concatenating (samples may have different extensions)
         # Without this, pl.concat fails with: ShapeError: unable to append DataFrame of width X with width Y
         aligned_dfs = align_dataframe_schemas(metadata_dfs)
         df = pl.concat(aligned_dfs, how="vertical")
-        
+
         return self._clean_dataframe(df)
 
     def _generate_nested_folders(
@@ -240,7 +230,9 @@ class MetadataGenerator:
         """Recursively generate local metadata for nested folders."""
         result = {}
 
-        for child in parent_sample.path.samples:
+        # Cast path to Tortilla to access .samples
+        tortilla = cast(Tortilla, parent_sample.path)
+        for child in tortilla.samples:
             if child.type == "FOLDER":
                 folder_path = f"{parent_path}{child.id}/"
                 folder_df = self._generate_folder_metadata(child)
@@ -252,124 +244,6 @@ class MetadataGenerator:
 
         return result
 
-    def _validate_pit_level0(self, df: pl.DataFrame) -> None:
-        """Validate PIT constraint for level 0 (root)."""
-        if "type" not in df.columns:
-            raise PITValidationError("Level 0 missing 'type' column")
-
-        types = df["type"].to_list()
-        unique_types = list(set(types))
-
-        if len(unique_types) != 1:
-            raise PITValidationError(
-                f"PIT constraint violated at level 0:\n"
-                f"All nodes must have the same type (all FILE or all FOLDER).\n"
-                f"Found types: {unique_types}"
-            )
-
-    def _validate_pit_depth(
-        self, df: pl.DataFrame, parent_df: pl.DataFrame, depth: int
-    ) -> None:
-        """Validate PIT constraints for hierarchical levels (depth 1+)."""
-        if "type" not in df.columns:
-            raise PITValidationError(f"Depth {depth} missing 'type' column")
-
-        parent_types = parent_df["type"].to_list()
-        parent_pattern = self._infer_unique_pattern(parent_types, depth - 1)
-        folder_positions = [i for i, t in enumerate(parent_pattern) if t == "FOLDER"]
-
-        if not folder_positions:
-            raise PITValidationError(
-                f"Depth {depth} exists but no FOLDERs at depth {depth - 1}"
-            )
-
-        num_parents = len(parent_df)
-        child_types = df["type"].to_list()
-
-        # Validate each folder position has consistent pattern
-        for folder_idx, position in enumerate(folder_positions):
-            chunk_pattern = self._extract_chunk_pattern(
-                child_types, num_parents, len(folder_positions), folder_idx
-            )
-
-            if chunk_pattern is None:
-                raise PITValidationError(
-                    f"PIT constraint violated at depth {depth}:\n"
-                    f"Cannot extract consistent pattern for FOLDER at position {position}"
-                )
-
-            # Validate pattern consistency across all parents
-            for parent_idx in range(num_parents):
-                chunk_start = (parent_idx * len(folder_positions) + folder_idx) * len(
-                    chunk_pattern
-                )
-                chunk_end = chunk_start + len(chunk_pattern)
-                actual_chunk = child_types[chunk_start:chunk_end]
-
-                if actual_chunk != chunk_pattern:
-                    raise PITValidationError(
-                        f"PIT constraint violated at depth {depth}:\n"
-                        f"FOLDER at position {position}, parent {parent_idx} has different pattern.\n"
-                        f"Expected: {chunk_pattern}\n"
-                        f"Actual: {actual_chunk}"
-                    )
-
-    def _infer_unique_pattern(self, types: list[str], depth: int) -> list[str]:
-        """Infer the repeating pattern of types at a level."""
-        total = len(types)
-
-        # Try to find shortest repeating pattern
-        for pattern_len in range(1, total // 2 + 1):
-            if total % pattern_len != 0:
-                continue
-
-            pattern = types[:pattern_len]
-            num_repeats = total // pattern_len
-
-            # Check if pattern repeats exactly
-            if all(
-                types[i * pattern_len : (i + 1) * pattern_len] == pattern
-                for i in range(num_repeats)
-            ):
-                return pattern
-
-        # No repeating pattern found - entire list is the pattern
-        return types
-
-    def _extract_chunk_pattern(
-        self,
-        types: list[str],
-        num_parents: int,
-        num_folders_per_parent: int,
-        folder_idx: int,
-    ) -> list[str] | None:
-        """Extract the pattern for a specific folder position."""
-        total_types = len(types)
-        expected_total = num_parents * num_folders_per_parent
-
-        if total_types % expected_total != 0:
-            return None
-
-        chunk_size = total_types // expected_total
-
-        # Extract pattern from first parent's folder at this position
-        first_chunk_start = folder_idx * chunk_size
-        first_chunk_end = first_chunk_start + chunk_size
-        pattern = types[first_chunk_start:first_chunk_end]
-
-        # Validate pattern is consistent across all parents
-        for parent_idx in range(num_parents):
-            for fld_idx in range(num_folders_per_parent):
-                if fld_idx == folder_idx:
-                    chunk_start = (
-                        parent_idx * num_folders_per_parent + fld_idx
-                    ) * chunk_size
-                    chunk_end = chunk_start + chunk_size
-                    if types[chunk_start:chunk_end] != pattern:
-                        return None
-
-        return pattern
-
     def _clean_dataframe(self, df: pl.DataFrame) -> pl.DataFrame:
         """Clean DataFrame by removing unnecessary columns."""
         # Remove path column (filesystem-specific, not container-relevant)
@@ -388,31 +262,75 @@ class MetadataGenerator:
         return df
 
 
-def generate_field_schema(levels: list[pl.DataFrame]) -> dict[str, Any]:
-    """Generate field schema describing columns at each level."""
+def generate_field_schema(levels: list[pl.DataFrame], taco: "Taco") -> dict[str, Any]:
+    """
+    Generate field schema with descriptions from extensions.
+
+    Collects field descriptions from all Sample and Tortilla extensions,
+    then generates arrays of [name, type, description] for each field.
+    If no description exists for a field, uses empty string.
+    """
+    from tacotoolbox._constants import (
+        CORE_FIELD_DESCRIPTIONS,
+        INTERNAL_FIELD_DESCRIPTIONS,
+    )
+
+    # Collect all field descriptions
+    all_descriptions: dict[str, str] = {}
+
+    # 1. Add core and internal field descriptions first
+    all_descriptions.update(CORE_FIELD_DESCRIPTIONS)
+    all_descriptions.update(INTERNAL_FIELD_DESCRIPTIONS)
+
+    # 2. From tortilla (overrides core/internal if redefined)
+    if hasattr(taco.tortilla, "_field_descriptions"):
+        all_descriptions.update(taco.tortilla._field_descriptions)
+
+    # 3. From samples recursively (overrides previous)
+    def collect_from_samples(samples: list["Sample"]) -> None:
+        for sample in samples:
+            if hasattr(sample, "_field_descriptions"):
+                all_descriptions.update(sample._field_descriptions)
+
+            # Recurse into FOLDER samples
+            if sample.type == "FOLDER":
+                tortilla_path = cast(Tortilla, sample.path)
+                if hasattr(tortilla_path, "samples"):
+                    collect_from_samples(tortilla_path.samples)
+
+    collect_from_samples(taco.tortilla.samples)
+
+    # Generate field schema with descriptions
     field_schema = {}
 
     for i, level_df in enumerate(levels):
         fields = []
         for col_name, col_type in level_df.schema.items():
             type_name = str(col_type).lower()
-            fields.append([col_name, type_name])
+            description = all_descriptions.get(col_name, "")
+            fields.append([col_name, type_name, description])
 
         field_schema[f"level{i}"] = fields
 
     return field_schema
 
 
-def generate_pit_schema(
+def generate_pit_schema(  # noqa: C901
     dataframes: list[pl.DataFrame], debug: bool = False
 ) -> dict[str, Any]:
-    """Generate Position-Isomorphic Tree (PIT) schema."""
+    """
+    Generate PIT schema.
+
+    NOTE: This function has high cyclomatic complexity (C901) but is intentionally
+    kept as a single function for algorithmic clarity. The PIT schema generation
+    is inherently complex and splitting it would reduce readability.
+    """
     if not dataframes:
-        raise PITValidationError("Need at least one DataFrame to generate schema")
+        raise ValueError("Need at least one DataFrame to generate schema")
 
     df0 = dataframes[0]
     if "type" not in df0.columns:
-        raise PITValidationError("Level 0 missing 'type' column")
+        raise ValueError("Level 0 missing 'type' column")
 
     # Root level
     root_type = df0["type"][0]
@@ -429,9 +347,7 @@ def generate_pit_schema(
             continue
 
         if METADATA_PARENT_ID not in df.columns:
-            raise PITValidationError(
-                f"Depth {depth} missing '{METADATA_PARENT_ID}' column"
-            )
+            raise ValueError(f"Depth {depth} missing '{METADATA_PARENT_ID}' column")
 
         if depth == 1:
             # LEVEL 1: Find folder with MOST real (non-padding) samples
@@ -551,4 +467,4 @@ def generate_collection_json(taco: "Taco") -> dict[str, Any]:
     """Generate COLLECTION.json content from TACO object."""
     collection = taco.model_dump()
     collection.pop("tortilla", None)
-    return collection
\ No newline at end of file
+    return collection
