@@ -11,19 +11,17 @@ Key features:
 - Recursive FOLDER handling with local metadata
 - Zero-copy data transfer via tacoreader.io
 - High-performance concurrent downloads with progress bars
-
-The ExportWriter uses async/await for optimal performance when downloading
-data from remote sources like S3 or HTTP endpoints. Local file I/O uses
-sync operations as they're fast and not the bottleneck.
 """
 
 import asyncio
 import json
 import pathlib
+import re
+from datetime import datetime, timezone
+from typing import Any, cast
 
 import polars as pl
 from tacoreader import TacoDataset
-from tacoreader.io import download_range
 from tacoreader.utils.vsi import parse_vsi_subfile, strip_vsi_prefix
 
 from tacotoolbox._column_utils import (
@@ -34,6 +32,7 @@ from tacotoolbox._column_utils import (
 from tacotoolbox._constants import FOLDER_DATA_DIR, FOLDER_METADATA_DIR
 from tacotoolbox._logging import get_logger
 from tacotoolbox._progress import progress_gather
+from tacotoolbox.remote_io import download_range
 
 logger = get_logger(__name__)
 
@@ -48,6 +47,30 @@ def _get_available_levels(dataset: TacoDataset) -> list[str]:
     return [f"level{i}" for i in range(max_depth + 1)]
 
 
+def _sanitize_sql_identifier(identifier: str) -> str:
+    """
+    Sanitize SQL table/view name to prevent injection.
+
+    Validates that identifier contains only alphanumeric characters and underscores.
+    Used for level names (level0, level1, etc.) to satisfy S608 checks.
+
+    Args:
+        identifier: SQL identifier to sanitize
+
+    Returns:
+        The same identifier if valid
+
+    Raises:
+        ValueError: If identifier contains invalid characters
+    """
+    if not re.match(r"^[a-zA-Z0-9_]+$", identifier):
+        raise ValueError(
+            f"Invalid SQL identifier: '{identifier}'. "
+            f"Must contain only alphanumeric characters and underscores."
+        )
+    return identifier
+
+
 class ExportWriter:
     """
     Handle creation of FOLDER containers from filtered TacoDataset.
@@ -61,9 +84,8 @@ class ExportWriter:
         self,
         dataset: TacoDataset,
         output: pathlib.Path,
-        concurrency: int = 100,
-        quiet: bool = False,
-        debug: bool = False,
+        limit: int = 100,
+        **kwargs,
     ) -> None:
         """
         Initialize export writer.
@@ -71,21 +93,18 @@ class ExportWriter:
         Args:
             dataset: TacoDataset with applied filters (e.g., from .sql())
             output: Path to output folder
-            concurrency: Maximum concurrent async operations (default: 100)
-            quiet: If True, hide progress bars (default: False - shows progress)
-            debug: If True, enable debug logging (default: False)
+            limit: Maximum concurrent async operations (default: 100)
+            **kwargs: Parquet config (compression, compression_level, row_group_size)
         """
         self.dataset = dataset
         self.output = pathlib.Path(output)
-        self.concurrency = concurrency
-        self.quiet = quiet
+        self.limit = limit
+        self.parquet_kwargs = kwargs
 
         self.data_dir = self.output / FOLDER_DATA_DIR
         self.metadata_dir = self.output / FOLDER_METADATA_DIR
 
-        logger.debug(
-            f"ExportWriter initialized: output={output}, concurrency={concurrency}"
-        )
+        logger.debug(f"ExportWriter initialized: output={output}, limit={limit}")
 
     async def create_folder(self) -> pathlib.Path:
         """
@@ -115,13 +134,13 @@ class ExportWriter:
             self._generate_consolidated_metadata()
             self._generate_collection_json()
 
+        except Exception:
+            logger.exception("Failed to export FOLDER")
+            raise
+        else:
             logger.info(f"FOLDER export complete: {self.output}")
             return self.output
 
-        except Exception as e:
-            logger.error(f"Failed to export FOLDER: {e}")
-            raise ExportWriterError(f"Failed to export FOLDER: {e}") from e
-
     def _validate_dataset(self) -> None:
         """
         Validate dataset before export.
@@ -130,10 +149,6 @@ class ExportWriter:
         - Dataset has no level1+ joins (only level0 filters supported)
         - Dataset is not empty (at least one sample)
         - Output path doesn't exist
-
-        Raises:
-            ValueError: If dataset validation fails
-            FileExistsError: If output exists
         """
         if self.dataset._has_level1_joins:
             raise ValueError(
@@ -169,7 +184,7 @@ class ExportWriter:
         files_df = df.filter(pl.col("type") == "FILE")
         folders_df = df.filter(pl.col("type") == "FOLDER")
 
-        semaphore = asyncio.Semaphore(self.concurrency)
+        semaphore = asyncio.Semaphore(self.limit)
 
         # Copy FILEs concurrently with progress bar
         if len(files_df) > 0:
@@ -179,17 +194,18 @@ class ExportWriter:
             for row in files_df.iter_rows(named=True):
                 vsi_path = row["internal:gdal_vsi"]
 
-                if "internal:relative_path" in row and row["internal:relative_path"]:
-                    relative_path = row["internal:relative_path"]
-                else:
-                    relative_path = row["id"]
+                relative_path = (
+                    row["internal:relative_path"]
+                    if row.get("internal:relative_path")
+                    else row["id"]
+                )
 
                 dest = self.data_dir / relative_path
 
                 task = self._copy_single_file(vsi_path, dest, semaphore)
                 tasks.append(task)
 
-            # Progress bar for FILE copying
+            # Progress bar controlled by logging level
             await progress_gather(
                 *tasks, desc="Copying FILEs", unit="file", colour="green"
             )
@@ -203,7 +219,7 @@ class ExportWriter:
                 task = self._copy_folder_recursive(row, level=0, semaphore=semaphore)
                 tasks.append(task)
 
-            # Progress bar for FOLDER copying
+            # Progress bar controlled by logging level
             await progress_gather(
                 *tasks, desc="Copying FOLDERs", unit="folder", colour="blue"
             )
@@ -218,13 +234,8 @@ class ExportWriter:
         - /vsisubfile/... paths (ZIP entries): Downloads bytes from offset/size
         - Regular filesystem paths: Direct file copy
 
-        Uses tacoreader.io for all remote downloads (S3/HTTP/GCS).
+        Uses tacotoolbox.remote_io for all remote downloads (S3/HTTP/GCS).
         Local file I/O uses sync operations as they're fast and not the bottleneck.
-
-        Args:
-            vsi_path: Source path (may be /vsisubfile/... or regular path)
-            dest_path: Destination path for copied file
-            semaphore: Asyncio semaphore to limit concurrency
         """
         async with semaphore:
             dest_path.parent.mkdir(parents=True, exist_ok=True)
@@ -240,7 +251,7 @@ class ExportWriter:
                         f.seek(offset)
                         data = f.read(size)
                 else:
-                    # Remote file: use tacoreader.io.download_range
+                    # Remote file: use tacotoolbox.remote_io.download_range
                     # Convert sync download_range to async with to_thread
                     data = await asyncio.to_thread(
                         download_range, clean_url, offset, size
@@ -258,7 +269,7 @@ class ExportWriter:
 
     async def _copy_folder_recursive(
         self,
-        folder_row: dict,
+        folder_row: dict[str, Any],
         level: int,
         semaphore: asyncio.Semaphore,
     ) -> None:
@@ -270,16 +281,10 @@ class ExportWriter:
         2. Query children from next level using parent_id
         3. Process all children concurrently (both FOLDERs and FILEs)
         4. Write local __meta__ file with children metadata (PARQUET format)
-
-        Args:
-            folder_row: Row from dataset.df for the FOLDER
-            level: Current level of this folder (0, 1, 2, ...)
-            semaphore: Asyncio semaphore to limit concurrency
         """
-        if level == 0:
-            relative_path = folder_row["id"]
-        else:
-            relative_path = folder_row["internal:relative_path"]
+        relative_path = (
+            folder_row["id"] if level == 0 else folder_row["internal:relative_path"]
+        )
 
         folder_path = self.data_dir / relative_path
         folder_path.mkdir(parents=True, exist_ok=True)
@@ -290,27 +295,29 @@ class ExportWriter:
         # Check if next level exists
         view_name = f"level{next_level}"
         available_levels = _get_available_levels(self.dataset)
-        
+
         if view_name not in available_levels:
             return
 
+        # Sanitize view_name before using in SQL
+        safe_view_name = _sanitize_sql_identifier(view_name)
+
         # Query children from next level
-        if "internal:source_file" in folder_row and folder_row["internal:source_file"]:
+        if folder_row.get("internal:source_file"):
             # TacoCat case: need to filter by both parent_id AND source_file
-            children_df = pl.from_arrow(
-                self.dataset._duckdb.execute(
-                    f'SELECT * FROM {view_name} WHERE "internal:parent_id" = ? AND "internal:source_file" = ?',
-                    [parent_id, folder_row["internal:source_file"]],
-                ).fetch_arrow_table()
-            )
+            arrow_table = self.dataset._duckdb.execute(
+                f'SELECT * FROM {safe_view_name} WHERE "internal:parent_id" = ? AND "internal:source_file" = ?',  # noqa: S608
+                [parent_id, folder_row["internal:source_file"]],
+            ).fetch_arrow_table()
         else:
             # Single TACO case: only filter by parent_id
-            children_df = pl.from_arrow(
-                self.dataset._duckdb.execute(
-                    f'SELECT * FROM {view_name} WHERE "internal:parent_id" = ?',
-                    [parent_id],
-                ).fetch_arrow_table()
-            )
+            arrow_table = self.dataset._duckdb.execute(
+                f'SELECT * FROM {safe_view_name} WHERE "internal:parent_id" = ?',  # noqa: S608
+                [parent_id],
+            ).fetch_arrow_table()
+
+        # Cast to DataFrame because pl.from_arrow returns DataFrame | Series
+        children_df = cast(pl.DataFrame, pl.from_arrow(arrow_table))
 
         # Process each child concurrently
         tasks = []
@@ -319,10 +326,7 @@ class ExportWriter:
             child_vsi = child_row["internal:gdal_vsi"]
 
             # Level 1+ has internal:relative_path, level 0 uses id
-            if (
-                "internal:relative_path" in child_row
-                and child_row["internal:relative_path"]
-            ):
+            if child_row.get("internal:relative_path"):
                 child_rel = child_row["internal:relative_path"]
             else:
                 child_rel = child_row["id"]
@@ -341,7 +345,7 @@ class ExportWriter:
 
         # Write local __meta__ for this folder in PARQUET format
         meta_path = folder_path / "__meta__"
-        write_parquet_file(children_df, meta_path)
+        write_parquet_file(children_df, meta_path, **self.parquet_kwargs)
 
     def _generate_consolidated_metadata(self) -> None:
         """
@@ -360,15 +364,19 @@ class ExportWriter:
         # Get available levels from pit_schema
         level_names = _get_available_levels(self.dataset)
 
-        parent_id_mapping = None
+        parent_id_mapping: dict[int, int] | None = None
 
         for level_name in level_names:
+            # Sanitize level_name before using in SQL
+            safe_level_name = _sanitize_sql_identifier(level_name)
+
             # Read from DuckDB instead of physical files
-            df = pl.from_arrow(
-                self.dataset._duckdb.execute(
-                    f"SELECT * FROM {level_name}"
-                ).fetch_arrow_table()
-            )
+            arrow_table = self.dataset._duckdb.execute(
+                f"SELECT * FROM {safe_level_name}"  # noqa: S608
+            ).fetch_arrow_table()
+
+            # Cast to DataFrame because pl.from_arrow returns DataFrame | Series
+            df = cast(pl.DataFrame, pl.from_arrow(arrow_table))
 
             if level_name == "level0":
                 # Filter to selected samples
@@ -387,6 +395,13 @@ class ExportWriter:
                 filtered_df = filtered_df.rechunk()
                 filtered_df = reorder_internal_columns(filtered_df)
             else:
+                # Replace assert with explicit raise
+                if parent_id_mapping is None:
+                    raise RuntimeError(
+                        "Logic error: level0 must be processed before sublevels. "
+                        "This indicates a bug in metadata generation ordering."
+                    )
+
                 # Filter to children of selected parents
                 old_parent_ids_to_keep = set(parent_id_mapping.keys())
                 filtered_df = df.filter(
@@ -412,7 +427,7 @@ class ExportWriter:
 
             # Write consolidated metadata as PARQUET with CDC
             output_path = self.metadata_dir / f"{level_name}.parquet"
-            write_parquet_file_with_cdc(filtered_df, output_path)
+            write_parquet_file_with_cdc(filtered_df, output_path, **self.parquet_kwargs)
 
             logger.debug(f"{level_name}.parquet: {len(filtered_df)} samples")
 
@@ -435,9 +450,6 @@ class ExportWriter:
 
         # Add subset provenance
         collection["taco:subset_of"] = collection.get("id", "unknown")
-
-        from datetime import datetime, timezone
-
         collection["taco:subset_date"] = datetime.now(timezone.utc).isoformat()
 
         # Write to file
@@ -445,4 +457,4 @@ class ExportWriter:
         with open(output_path, "w", encoding="utf-8") as f:
             json.dump(collection, f, indent=4, ensure_ascii=False)
 
-        logger.debug("COLLECTION.json created")
\ No newline at end of file
+        logger.debug("COLLECTION.json created")
