@@ -1,5 +1,9 @@
-from tacotoolbox.taco.extensions.labels import Labels, LabelClass
-from tacotoolbox.taco.extensions.opticaldata import SpectralBand, OpticalData, SUPPORTED_SENSORS
+from tacotoolbox.taco.datamodel import Contact, Extent
+from tacotoolbox.taco.extensions.labels import LabelClass, Labels
+from tacotoolbox.taco.extensions.opticaldata import (
+    SUPPORTED_SENSORS,
+    OpticalData,
+    SpectralBand,
+)
 from tacotoolbox.taco.extensions.scientific import Publication, Publications
 from tacotoolbox.taco.extensions.split import SplitStrategy, SplitStrategyType
-from tacotoolbox.taco.datamodel import Contact, Extent
\ No newline at end of file
