@@ -12,6 +12,7 @@ Key features:
 - Automatic cleanup of temporary files from bytes
 """
 
+import contextlib
 import pathlib
 import re
 import tempfile
@@ -23,7 +24,7 @@ from typing import TYPE_CHECKING, Any, Literal
 import polars as pl
 import pydantic
 
-from tacotoolbox._constants import SHARED_CORE_FIELDS, SHARED_PROTECTED_FIELDS
+from tacotoolbox._constants import SHARED_CORE_FIELDS
 from tacotoolbox.tortilla.datamodel import Tortilla
 
 if TYPE_CHECKING:
@@ -42,7 +43,7 @@ class SampleExtension(ABC, pydantic.BaseModel):
     schema_only: bool = pydantic.Field(
         False,
         description="If True, return None values while preserving schema",
-        validation_alias="return_none"
+        validation_alias="return_none",
     )
 
     @abstractmethod
@@ -50,6 +51,11 @@ class SampleExtension(ABC, pydantic.BaseModel):
         """Return the expected schema for this extension."""
         pass
 
+    @abstractmethod
+    def get_field_descriptions(self) -> dict[str, str]:
+        """Return field descriptions for each field."""
+        pass
+
     @abstractmethod
     def _compute(self, sample: "Sample") -> pl.DataFrame:
         """Actual computation logic - only called when schema_only=False."""
@@ -114,12 +120,15 @@ class Sample(pydantic.BaseModel):
         default_factory=dict
     )
 
+    # Private attribute to store field descriptions
+    _field_descriptions: dict[str, str] = pydantic.PrivateAttr(default_factory=dict)
+
     model_config = pydantic.ConfigDict(
         arbitrary_types_allowed=True,  # Support Tortilla dataclass
         extra="allow",  # Allow dynamic fields from extensions
     )
 
-    def __init__(self, temp_dir: pathlib.Path | None = None, **data):
+    def __init__(self, temp_dir: pathlib.Path | None = None, **data: Any) -> None:
         """Initialize Sample with optional temp_dir for bytes conversion."""
         # Handle temp_dir for bytes conversion without storing it
         if "path" in data and isinstance(data["path"], bytes):
@@ -149,8 +158,9 @@ class Sample(pydantic.BaseModel):
             object.__setattr__(self, "_temp_files", [temp_path])
 
         # Extract extension fields (anything not a core field)
-        core_fields = SHARED_CORE_FIELDS | {"type"}  # Include 'type' as core
-        extension_fields = {k: v for k, v in data.items() if k not in core_fields}
+        extension_fields = {
+            k: v for k, v in data.items() if k not in SHARED_CORE_FIELDS
+        }
 
         # Initialize with all fields (Pydantic accepts them due to extra="allow")
         super().__init__(**data)
@@ -194,6 +204,7 @@ class Sample(pydantic.BaseModel):
         # Manually initialize private attributes (model_construct doesn't do this)
         object.__setattr__(sample, "_temp_files", [temp_path])
         object.__setattr__(sample, "_extension_schemas", {})
+        object.__setattr__(sample, "_field_descriptions", {})
 
         return sample
 
@@ -207,22 +218,16 @@ class Sample(pydantic.BaseModel):
             return
 
         for temp_file in self._temp_files:
-            try:
+            with contextlib.suppress(Exception):
                 if temp_file.exists():
                     temp_file.unlink()
-            except Exception:
-                # Silently ignore cleanup errors
-                pass
 
         self._temp_files.clear()
 
-    def __del__(self):
+    def __del__(self) -> None:
         """Cleanup on garbage collection."""
-        try:
+        with contextlib.suppress(Exception):
             self.cleanup()
-        except Exception:
-            # Silently ignore cleanup errors during garbage collection
-            pass
 
     @pydantic.field_validator("id")
     @classmethod
@@ -277,7 +282,7 @@ class Sample(pydantic.BaseModel):
         )
 
     @pydantic.model_validator(mode="after")
-    def infer_and_validate_type(self):
+    def infer_and_validate_type(self) -> "Sample":
         """
         Infer type from path or validate explicit type.
 
@@ -334,7 +339,7 @@ class Sample(pydantic.BaseModel):
 
         return None
 
-    def _handle_sample_extension(self, extension) -> None:
+    def _handle_sample_extension(self, extension: Any) -> None:
         """Handle SampleExtension (callable with model_dump)."""
         computed_metadata = extension(self)
         if isinstance(computed_metadata, pl.DataFrame):
@@ -346,6 +351,11 @@ class Sample(pydantic.BaseModel):
             for col_name, dtype in computed_metadata.schema.items():
                 self._extension_schemas[col_name] = dtype
 
+            # Capture field descriptions if extension provides them
+            if hasattr(extension, "get_field_descriptions"):
+                descriptions = extension.get_field_descriptions()
+                self._field_descriptions.update(descriptions)
+
             metadata_dict = computed_metadata.to_dicts()[0]
             self._add_metadata_fields(metadata_dict)
 
@@ -361,7 +371,7 @@ class Sample(pydantic.BaseModel):
         metadata_dict = extension.to_dicts()[0]
         self._add_metadata_fields(metadata_dict)
 
-    def _handle_dict_extension(self, extension: dict) -> None:
+    def _handle_dict_extension(self, extension: dict[str, Any]) -> None:
         """Handle dictionary extension."""
         # First, infer schemas from dict values and update _extension_schemas
         for key, value in extension.items():
@@ -371,7 +381,7 @@ class Sample(pydantic.BaseModel):
         # Then add the fields
         self._add_metadata_fields(extension)
 
-    def _handle_pydantic_extension(self, extension, name: str | None) -> None:
+    def _handle_pydantic_extension(self, extension: Any, name: str | None) -> None:
         """Handle Pydantic model extension."""
         namespace = name if name else extension.__class__.__name__.lower()
         if hasattr(extension, "model_dump"):
@@ -386,11 +396,11 @@ class Sample(pydantic.BaseModel):
                 f"Extension must be pydantic model or dict, got: {type(extension)}"
             )
 
-    def _add_metadata_fields(self, metadata_dict: dict) -> None:
+    def _add_metadata_fields(self, metadata_dict: dict[str, Any]) -> None:
         """Add metadata fields to the sample with validation."""
         for key, value in metadata_dict.items():
             self._validate_key(key)
-            if key in SHARED_PROTECTED_FIELDS:
+            if key in SHARED_CORE_FIELDS:
                 raise ValueError(f"Cannot override core field: {key}")
             setattr(self, key, value)
 
@@ -464,4 +474,4 @@ class Sample(pydantic.BaseModel):
         # Create DataFrame with explicit schema
         df = pl.DataFrame([data], schema=complete_schema)
 
-        return df
\ No newline at end of file
+        return df
