@@ -11,16 +11,39 @@ Main components:
 - Contact: Contributor information
 - Extent: Spatial/temporal boundaries
 - TaskType: ML task categories
+
+Positional-Invariance Tree (PIT):
+    PIT validates that all root samples are isomorphic. For hierarchical datasets
+    (all FOLDER samples), this means:
+    - Same number of children at each position
+    - Same ID at each position across all roots
+    - Same type at each position across all roots
+    - Recursive validation for nested FOLDERs
+
+    Rule: Root[i].children[k] must be identical for all i at position k.
+
+    Example (valid):
+        Date_0/ -> [GLORYS/, L4/, L3/]
+        Date_1/ -> [GLORYS/, L4/, L3/]
+        Date_2/ -> [GLORYS/, L4/, L3/]
+
+    Example (invalid):
+        Date_0/ -> [GLORYS/, L4/]
+        Date_1/ -> [L4/, GLORYS/]  # Position 0 must be GLORYS
+
+    Validation occurs at Taco creation to ensure structural consistency.
 """
 
 from abc import ABC, abstractmethod
 from datetime import datetime, timezone
-from typing import Literal
+from typing import Any, Literal, cast
 
 import polars as pl
 import pydantic
+from shapely.geometry import Point
 from shapely.wkb import loads as wkb_loads
 
+from tacotoolbox._constants import PADDING_PREFIX
 from tacotoolbox.tortilla.datamodel import Tortilla
 
 
@@ -32,6 +55,11 @@ class TacoExtension(ABC, pydantic.BaseModel):
         """Return column names and Polars DataTypes for this extension."""
         pass
 
+    @abstractmethod
+    def get_field_descriptions(self) -> dict[str, str]:
+        """Return field descriptions for each field."""
+        pass
+
     @abstractmethod
     def _compute(self, taco: "Taco") -> pl.DataFrame:
         """Compute extension metadata, return single-row DataFrame."""
@@ -82,7 +110,7 @@ class Contact(TacoExtension):
     role: str | None = None
 
     @pydantic.model_validator(mode="after")
-    def check_name_or_organization(self):
+    def check_name_or_organization(self) -> "Contact":
         if not self.name and not self.organization:
             raise ValueError("Either 'name' or 'organization' must be provided")
         return self
@@ -102,6 +130,14 @@ class Contact(TacoExtension):
             "role": pl.Utf8(),
         }
 
+    def get_field_descriptions(self) -> dict[str, str]:
+        return {
+            "name": "Contributor name",
+            "organization": "Contributor organization or institution",
+            "email": "Contact email address",
+            "role": "Contributor role (e.g., principal-investigator, data-curator, maintainer)",
+        }
+
     def _compute(self, taco: "Taco") -> pl.DataFrame:
         return pl.DataFrame([self.model_dump()])
 
@@ -179,6 +215,12 @@ class Extent(TacoExtension):
     def get_schema(self) -> dict[str, pl.DataType]:
         return {"spatial": pl.List(pl.Float64()), "temporal": pl.List(pl.Utf8())}
 
+    def get_field_descriptions(self) -> dict[str, str]:
+        return {
+            "spatial": "Spatial bounding box [min_lon, min_lat, max_lon, max_lat] in WGS84 decimal degrees",
+            "temporal": "Temporal interval [start_datetime, end_datetime] in ISO 8601 format",
+        }
+
     def _compute(self, taco: "Taco") -> pl.DataFrame:
         return pl.DataFrame([self.model_dump()])
 
@@ -250,7 +292,123 @@ class Taco(pydantic.BaseModel):
         return value
 
     @pydantic.model_validator(mode="after")
-    def auto_calculate_extent(self) -> "Taco":
+    def validate_and_initialize(self) -> "Taco":
+        """Validate PIT compliance and auto-calculate extent."""
+        self._validate_pit_compliance()
+        self._auto_calculate_extent()
+        return self
+
+    def _validate_pit_compliance(self) -> None:  # noqa: C901
+        """
+        Validate PIT constraint for root tortilla.
+
+        This is the only place where PIT validation occurs because Taco knows
+        its tortilla is the root of the dataset tree. Validates that all root
+        samples are structurally isomorphic.
+        """
+        samples = self.tortilla.samples
+
+        # Validate all root samples have same type
+        root_types = [s.type for s in samples]
+        unique_types = set(root_types)
+
+        if len(unique_types) > 1:
+            raise ValueError(
+                f"PIT violation: All root samples must have the same type.\n"
+                f"Found mixed types at root level: {unique_types}\n"
+                f"All root samples must be either all FILE or all FOLDER."
+            )
+
+        # If all FILEs, no further validation needed
+        if "FOLDER" not in unique_types:
+            return
+
+        # Collect FOLDER samples
+        folder_samples = [s for s in samples if s.type == "FOLDER"]
+
+        if len(folder_samples) <= 1:
+            return
+
+        # Get non-padding children from each folder
+        folder_children: list[tuple[str, list]] = []
+        for folder in folder_samples:
+            if hasattr(folder.path, "samples") and folder.path.samples:
+                real_children = [
+                    child
+                    for child in folder.path.samples
+                    if not child.id.startswith(PADDING_PREFIX)
+                ]
+                folder_children.append((folder.id, real_children))
+
+        if len(folder_children) <= 1:
+            return
+
+        # Validate all folders have same number of children
+        child_counts = [len(children) for _, children in folder_children]
+        if len(set(child_counts)) > 1:
+            count_report = ", ".join(
+                f"'{folder_id}': {count}"
+                for (folder_id, _), count in zip(
+                    folder_children, child_counts, strict=False
+                )
+            )
+            raise ValueError(
+                f"PIT violation: All root samples must be isomorphic.\n"
+                f"Child counts (excluding padding): {count_report}\n"
+                f"All root samples must have the same internal structure."
+            )
+
+        # Validate PIT
+        reference_folder_id, reference_children = folder_children[0]
+        num_positions = len(reference_children)
+
+        for pos in range(num_positions):
+            reference_child = reference_children[pos]
+            reference_id = reference_child.id
+            reference_type = reference_child.type
+
+            for folder_id, children in folder_children[1:]:
+                current_child = children[pos]
+
+                if current_child.id != reference_id:
+                    raise ValueError(
+                        f"PIT violation at position {pos}:\n"
+                        f"Root sample '{reference_folder_id}' has child ID: '{reference_id}'\n"
+                        f"Root sample '{folder_id}' has child ID: '{current_child.id}'\n"
+                        f"All root samples must have the same child ID at position {pos}."
+                    )
+
+                if current_child.type != reference_type:
+                    raise ValueError(
+                        f"PIT violation at position {pos} (ID: '{reference_id}'):\n"
+                        f"Root sample '{reference_folder_id}' has child type: '{reference_type}'\n"
+                        f"Root sample '{folder_id}' has child type: '{current_child.type}'\n"
+                        f"All root samples must have the same child type at position {pos}."
+                    )
+
+        # Recursive validation for nested FOLDERs
+        for pos in range(num_positions):
+            children_at_position = [children[pos] for _, children in folder_children]
+
+            if all(child.type == "FOLDER" for child in children_at_position):
+                try:
+                    child_tortillas = [
+                        cast(Tortilla, child.path) for child in children_at_position
+                    ]
+                    for i, child_tortilla in enumerate(child_tortillas[1:], start=1):
+                        if len(child_tortilla.samples) != len(
+                            child_tortillas[0].samples
+                        ):
+                            raise ValueError(
+                                f"PIT violation in nested structure at position {pos}:\n"
+                                f"Root[0].child[{pos}] ('{children_at_position[0].id}') has {len(child_tortillas[0].samples)} children\n"
+                                f"Root[{i}].child[{pos}] ('{children_at_position[i].id}') has {len(child_tortilla.samples)} children\n"
+                                f"All children at position {pos} must be isomorphic."
+                            )
+                except AttributeError:
+                    pass
+
+    def _auto_calculate_extent(self) -> None:
         """
         Calculate extent automatically from STAC/ISTAC metadata in samples.
 
@@ -301,12 +459,11 @@ class Taco(pydantic.BaseModel):
                     )
 
                 self.extent = Extent(spatial=spatial, temporal=temporal)
-                return self
+                return
 
         self.extent = Extent(spatial=[-180.0, -90.0, 180.0, 90.0], temporal=None)
-        return self
 
-    def extend_with(self, extension):
+    def extend_with(self, extension: Any) -> None:
         """
         Add extension data to dataset.
 
@@ -346,11 +503,11 @@ class Taco(pydantic.BaseModel):
         extension_data = extension.to_dicts()[0]
         self._set_extension_attributes(extension_data)
 
-    def _handle_dict_extension(self, extension: dict) -> None:
+    def _handle_dict_extension(self, extension: dict[str, Any]) -> None:
         """Handle dictionary extension."""
         self._set_extension_attributes(extension)
 
-    def _handle_pydantic_extension(self, extension) -> None:
+    def _handle_pydantic_extension(self, extension: Any) -> None:
         """Handle Pydantic model extension."""
         if hasattr(extension, "model_dump"):
             extension_data = extension.model_dump()
@@ -360,7 +517,7 @@ class Taco(pydantic.BaseModel):
                 "Extension must be TacoExtension, DataFrame, dict, or pydantic model"
             )
 
-    def _set_extension_attributes(self, extension_data: dict) -> None:
+    def _set_extension_attributes(self, extension_data: dict[str, Any]) -> None:
         """Set extension fields as instance attributes."""
         for key, value in extension_data.items():
             setattr(self, key, value)
@@ -372,13 +529,13 @@ class Taco(pydantic.BaseModel):
         Preserves nested structures (Contact, Extent) without flattening.
         Includes core fields, Tortilla reference, and all extensions.
         """
-        metadata: dict = {}
+        metadata: dict[str, Any] = {}
 
         for key, value in self.__dict__.items():
             if key.startswith("_"):
                 continue
 
-            if isinstance(value, Contact) or isinstance(value, Extent):
+            if isinstance(value, Contact | Extent):
                 metadata[key] = value.model_dump()
 
             elif isinstance(value, list) and value and isinstance(value[0], Contact):
@@ -398,7 +555,9 @@ def _calculate_spatial_extent(df: pl.DataFrame, centroid_col: str) -> list[float
     Skips None values (padding samples).
     """
     centroids = [
-        wkb_loads(wkb) for wkb in df[centroid_col].to_list() if wkb is not None
+        cast(Point, wkb_loads(wkb))
+        for wkb in df[centroid_col].to_list()
+        if wkb is not None
     ]
 
     if not centroids:
@@ -417,34 +576,46 @@ def _calculate_temporal_extent(
     time_middle_col: str | None,
 ) -> list[str] | None:
     """
-    Calculate temporal interval from timestamp columns.
+    Calculate temporal interval from Datetime columns.
 
-    Converts integer timestamps to ISO 8601 datetime strings.
+    Extracts datetime values, computes min/max in Python.
     Skips None values (padding samples).
 
     Priority cascade: time_middle > time_start > time_end
+    Tries each column until finding one with valid values.
     """
-    time_values = []
-
+    # Try columns in priority order until finding one with valid values
+    candidate_cols = []
     if time_middle_col and time_middle_col in df.columns:
-        time_values = [t for t in df[time_middle_col].to_list() if t is not None]
+        candidate_cols.append(time_middle_col)
+    if time_start_col in df.columns:
+        candidate_cols.append(time_start_col)
+    if time_end_col and time_end_col in df.columns:
+        candidate_cols.append(time_end_col)
 
-    if not time_values:
-        time_values = [t for t in df[time_start_col].to_list() if t is not None]
-
-    if not time_values and time_end_col and time_end_col in df.columns:
-        time_values = [t for t in df[time_end_col].to_list() if t is not None]
-
-    if not time_values:
+    if not candidate_cols:
         return None
 
-    min_time = min(time_values)
-    max_time = max(time_values)
+    # Try each candidate until finding one with non-None values
+    for time_col in candidate_cols:
+        time_values = [dt for dt in df[time_col].to_list() if dt is not None]
 
-    start_dt = datetime.fromtimestamp(min_time, tz=timezone.utc)
-    end_dt = datetime.fromtimestamp(max_time, tz=timezone.utc)
+        if time_values:
+            # Found valid values, compute extent
+            min_dt = min(time_values)
+            max_dt = max(time_values)
 
-    return [
-        start_dt.isoformat().replace("+00:00", "Z"),
-        end_dt.isoformat().replace("+00:00", "Z"),
-    ]
\ No newline at end of file
+            # Convert to UTC if not already (Polars Datetime without timezone is naive)
+            if min_dt.tzinfo is None:
+                min_dt = min_dt.replace(tzinfo=timezone.utc)
+            if max_dt.tzinfo is None:
+                max_dt = max_dt.replace(tzinfo=timezone.utc)
+
+            # Convert to ISO 8601 strings with 'Z' suffix
+            return [
+                min_dt.isoformat().replace("+00:00", "Z"),
+                max_dt.isoformat().replace("+00:00", "Z"),
+            ]
+
+    # No column had valid values
+    return None
