@@ -6,13 +6,12 @@ Supports nested sample structures with position tracking.
 
 Key features:
 - Hierarchical metadata export (up to SHARED_MAX_DEPTH levels)
-- Position-Isomorphic Tree (PIT) validation
 - Auto-padding for homogeneous structures
 - Efficient DataFrame construction with schema validation
 - Configurable schema strictness (strict_schema parameter)
 
-Best Practice: When mixing FOLDERs and FILEs, place all FOLDERs before FILEs
-for better organization and readability.
+Best Practice: When mixing FOLDERs and FILEs in the same Tortilla,
+place all FOLDERs before FILEs for better organization and readability.
 """
 
 import warnings
@@ -27,8 +26,8 @@ from tacotoolbox._column_utils import align_dataframe_schemas
 from tacotoolbox._constants import (
     METADATA_PARENT_ID,
     PADDING_PREFIX,
-    validate_depth,
 )
+from tacotoolbox._utils import validate_depth
 
 if TYPE_CHECKING:
     from tacotoolbox.sample.datamodel import Sample
@@ -48,6 +47,11 @@ class TortillaExtension(ABC, pydantic.BaseModel):
         """Return the expected schema for this extension."""
         pass
 
+    @abstractmethod
+    def get_field_descriptions(self) -> dict[str, str]:
+        """Return field descriptions for each field."""
+        pass
+
     @abstractmethod
     def _compute(self, tortilla: "Tortilla") -> pl.DataFrame:
         """Actual computation logic - only called when schema_only=False."""
@@ -96,7 +100,7 @@ class Tortilla:
 
         Raises:
             ValueError: If samples have inconsistent metadata schemas (strict_schema=True)
-                       or empty list or if Inclusive ID Rule is violated or duplicate IDs
+                       or empty list or duplicate IDs
         """
         if not samples:
             raise ValueError("Cannot create Tortilla with empty samples list")
@@ -107,20 +111,24 @@ class Tortilla:
 
         self.samples = samples
 
+        # Initialize field descriptions tracker
+        self._field_descriptions: dict[str, str] = {}
+
         # Validate unique IDs at this level
         self._validate_unique_ids()
 
         # Check ordering best practice (only for mixed types)
         self._check_sample_ordering()
 
-        # Validate Inclusive ID Rule
-        self._validate_inclusive_ids()
-
         # Extract metadata from all samples
         metadata_dfs = []
         for sample in samples:
             metadata_dfs.append(sample.export_metadata())
 
+            # Collect field descriptions from samples
+            if hasattr(sample, "_field_descriptions"):
+                self._field_descriptions.update(sample._field_descriptions)
+
         # Handle schema validation/alignment
         if strict_schema:
             # STRICT MODE: All schemas must match exactly
@@ -134,8 +142,7 @@ class Tortilla:
         else:
             # FLEXIBLE MODE: Use helper function to align schemas
             metadata_dfs = align_dataframe_schemas(
-                metadata_dfs, 
-                core_fields=["id", "type", "path"]
+                metadata_dfs, core_fields=["id", "type", "path"]
             )
 
         # Concatenate DataFrames - all schemas are guaranteed consistent
@@ -153,9 +160,7 @@ class Tortilla:
         missing_columns = reference_columns - current_columns
         extra_columns = current_columns - reference_columns
 
-        error_msg = (
-            f"Schema inconsistency detected at sample {sample_index} (id: '{sample.id}'):\n\n"
-        )
+        error_msg = f"Schema inconsistency detected at sample {sample_index} (id: '{sample.id}'):\n\n"
 
         error_msg += (
             f"  Reference sample columns: {sorted(reference_columns)}\n"
@@ -170,7 +175,7 @@ class Tortilla:
 
         error_msg += (
             "\n"
-            "  💡 Hint: If you want to combine samples with different schemas, use:\n"
+            "  Hint: If you want to combine samples with different schemas, use:\n"
             "      Tortilla(samples=[...], strict_schema=False)\n\n"
             "  This will auto-fill missing columns with None values.\n\n"
             "  To inspect sample schemas before creating Tortilla:\n"
@@ -180,9 +185,7 @@ class Tortilla:
 
         raise ValueError(error_msg)
 
-    def _align_schema(
-        self, df: pl.DataFrame, target_columns: set[str]
-    ) -> pl.DataFrame:
+    def _align_schema(self, df: pl.DataFrame, target_columns: set[str]) -> pl.DataFrame:
         """
         Align DataFrame schema to match target columns by adding missing columns with None.
         """
@@ -208,12 +211,15 @@ class Tortilla:
         resulting in missing files in the final container.
         """
         ids = [s.id for s in self.samples]
-        duplicates = {id: count for id, count in Counter(ids).items() if count > 1}
+        duplicates = {
+            sample_id: count for sample_id, count in Counter(ids).items() if count > 1
+        }
 
         if duplicates:
             # Show first 10 duplicates for debugging
             dup_list = ", ".join(
-                f"'{id}' ({count}x)" for id, count in list(duplicates.items())[:10]
+                f"'{sample_id}' ({count}x)"
+                for sample_id, count in list(duplicates.items())[:10]
             )
 
             raise ValueError(
@@ -290,90 +296,17 @@ class Tortilla:
             (i for i, sample in enumerate(self.samples) if sample.type == "FILE"), None
         )
 
-        if first_file_idx is not None and first_folder_idx is not None:
-            if first_file_idx < first_folder_idx:
-                warnings.warn(
-                    f"Consider placing FOLDERs before FILEs for better organization. "
-                    f"Found FILE at position {first_file_idx} before FOLDER at position {first_folder_idx}.",
-                    UserWarning,
-                    stacklevel=3,
-                )
-
-    def _validate_inclusive_ids(self) -> None:
-        """
-        Validate Inclusive ID Rule: The folder with maximum non-padding IDs
-        must contain ALL IDs from folders with fewer IDs (subset relationship).
-
-        IDs starting with '__TACOPAD__' are IGNORED in validation as they
-        represent padding placeholders, not real data.
-
-        This ensures that dataset.read("some_id") only fails if that ID
-        doesn't exist in ANY folder, not just some.
-        """
-        # Collect FOLDER samples
-        folder_samples = [s for s in self.samples if s.type == "FOLDER"]
-
-        # Skip validation if no folders or only one folder
-        if len(folder_samples) <= 1:
-            return
-
-        # Extract NON-PADDING child ID sets from each FOLDER
-        id_sets: list[tuple[str, frozenset[str]]] = []
-
-        for folder in folder_samples:
-            if hasattr(folder.path, "samples") and folder.path.samples:
-                real_ids = frozenset(
-                    sample.id
-                    for sample in folder.path.samples
-                    if not sample.id.startswith(PADDING_PREFIX)
-                )
-                id_sets.append((folder.id, real_ids))
-
-        # If only one folder has children, skip
-        if len(id_sets) <= 1:
-            return
-
-        # Find the folder with maximum IDs (the superset)
-        max_folder_id, max_id_set = max(id_sets, key=lambda x: len(x[1]))
-
-        # Validate: ALL other folders must be SUBSETS of max_id_set
-        for folder_id, child_set in id_sets:
-            if folder_id == max_folder_id:
-                continue
-
-            if not child_set.issubset(max_id_set):
-                # Found IDs that are NOT in the maximum set
-                extra_ids = child_set - max_id_set
-
-                # Build error message
-                error_parts = [
-                    "Inclusive ID Rule violated:",
-                    "The folder with maximum IDs must contain ALL IDs from other folders.",
-                    "(IDs starting with '__TACOPAD__' are ignored in validation)",
-                    "",
-                    f"Maximum ID folder: '{max_folder_id}'",
-                    f"  Non-padding IDs ({len(max_id_set)}): {sorted(max_id_set)}",
-                    "",
-                    f"Problematic folder: '{folder_id}'",
-                    f"  Non-padding IDs ({len(child_set)}): {sorted(child_set)}",
-                    "",
-                    f"IDs in '{folder_id}' NOT in '{max_folder_id}': {sorted(extra_ids)}",
-                    "",
-                    "How to fix:",
-                    "1. Ensure the folder with most files contains ALL possible IDs",
-                    "2. Other folders can have fewer files (will be padded automatically)",
-                    "3. Use Tortilla(samples, pad_to=N) to fill missing slots with '__TACOPAD__'",
-                    "",
-                    "Valid example:",
-                    "  Folder A: ['cloudmask', 'data', 'thumbnail']  # Maximum",
-                    "  Folder B: ['cloudmask', 'data']  # Subset of A, OK",
-                    "",
-                    "Invalid example:",
-                    "  Folder A: ['cloudmask', 'data']",
-                    "  Folder B: ['cloudmask', 'thumbnail']  # 'thumbnail' not in A, FAIL",
-                ]
-
-                raise ValueError("\n".join(error_parts))
+        if (
+            first_file_idx is not None
+            and first_folder_idx is not None
+            and first_file_idx < first_folder_idx
+        ):
+            warnings.warn(
+                f"Consider placing FOLDERs before FILEs for better organization. "
+                f"Found FILE at position {first_file_idx} before FOLDER at position {first_folder_idx}.",
+                UserWarning,
+                stacklevel=3,
+            )
 
     def _calculate_current_depth(self) -> int:
         """Calculate current depth by examining samples."""
@@ -415,6 +348,11 @@ class Tortilla:
         if conflicts:
             raise ValueError(f"Column conflicts: {sorted(conflicts)}")
 
+        # Capture field descriptions if extension provides them
+        if hasattr(extension, "get_field_descriptions"):
+            descriptions = extension.get_field_descriptions()
+            self._field_descriptions.update(descriptions)
+
         # Horizontal concatenation in polars using hstack
         self._metadata_df = self._metadata_df.hstack(result_df)
         return self
@@ -462,39 +400,35 @@ class Tortilla:
             next_dfs = []
             next_samples = []
 
-            # Track cumulative global index across all parents
-            # Ensures parent_id references the correct row in consolidated DataFrame
-            global_parent_idx = 0
+            # Use enumerate instead of manual counter
+            for global_parent_idx, sample in enumerate(current_samples):
+                if sample.type == "FOLDER":
+                    # Cast to Tortilla to ensure mypy knows it has .samples
+                    tortilla_path = cast(Tortilla, sample.path)
 
-            for sample in current_samples:
-                if sample.type == "FOLDER" and sample.path.samples:
-                    # Sample has children - process them
-                    for child_sample in sample.path.samples:
-                        child_metadata_df = child_sample.export_metadata()
+                    if tortilla_path.samples:
+                        # Sample has children - process them
+                        for child_sample in tortilla_path.samples:
+                            child_metadata_df = child_sample.export_metadata()
 
-                        # Use Int64 for parent_id consistency across ALL levels
-                        # This ensures level0 (Int64) and level1+ (Int64) match for JOINs
-                        child_metadata_df = child_metadata_df.with_columns(
-                            pl.lit(global_parent_idx, dtype=pl.Int64).alias(
-                                METADATA_PARENT_ID
+                            # Use Int64 for parent_id consistency across ALL levels
+                            # This ensures level0 (Int64) and level1+ (Int64) match for JOINs
+                            child_metadata_df = child_metadata_df.with_columns(
+                                pl.lit(global_parent_idx, dtype=pl.Int64).alias(
+                                    METADATA_PARENT_ID
+                                )
                             )
-                        )
 
-                        next_dfs.append(child_metadata_df)
-                        next_samples.append(child_sample)
-
-                # Increment for every parent sample (FILE or FOLDER)
-                # Ensures parent_id mapping stays aligned with parent DataFrame indices
-                global_parent_idx += 1
+                            next_dfs.append(child_metadata_df)
+                            next_samples.append(child_sample)
 
             current_dfs = next_dfs
             current_samples = next_samples
 
-        # Align schemas before concatenating using DRY helper function        
+        # Align schemas before concatenating using DRY helper function
         if len(current_dfs) > 1:
             current_dfs = align_dataframe_schemas(
-                current_dfs,
-                core_fields=["id", "type", "path", METADATA_PARENT_ID]
+                current_dfs, core_fields=["id", "type", "path", METADATA_PARENT_ID]
             )
 
-        return pl.concat(current_dfs, how="vertical")
\ No newline at end of file
+        return pl.concat(current_dfs, how="vertical")
