@@ -1,25 +1,48 @@
 """
-GeoEnrich extension for Tortilla.
+GeoEnrich extension for Tortilla using Earth Engine.
 
-Enriches samples with geospatial data from Earth Engine:
+Enriches samples with geospatial data from Google Earth Engine:
 - Physical: elevation, topographic complexity
 - Climate: precipitation, temperature
 - Soil: clay, sand, carbon, bulk density, pH
-- Socioeconomic: GDP, population, human modification
-- Administrative: countries, states, districts
+- Socioeconomic: GDP, population density, human modification index
+- Administrative: countries, states, districts (human-readable names)
 
-Single-file architecture with clear sections for maintainability.
+Fetches data at sample centroids using Earth Engine reducers.
+Uses spatial sorting (Morton encoding) for improved EE cache locality.
+
+Exports to Polars DataFrame:
+- geoenrich:elevation: Float32
+- geoenrich:cisi: Float32
+- geoenrich:precipitation: Float32
+- geoenrich:temperature: Float32
+- geoenrich:soil_clay: Float32
+- geoenrich:soil_sand: Float32
+- geoenrich:soil_carbon: Float32
+- geoenrich:soil_bulk_density: Float32
+- geoenrich:soil_ph: Float32
+- geoenrich:gdp: Float32
+- geoenrich:human_modification: Float32
+- geoenrich:population: Float32
+- geoenrich:admin_countries: String
+- geoenrich:admin_states: String
+- geoenrich:admin_districts: String
+
+Requirements:
+- earthengine-api: pip install earthengine-api
+- earthengine authenticate (one-time setup)
 """
 
 from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from functools import lru_cache
 from importlib.resources import as_file, files
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, cast
 
 import polars as pl
 import pydantic
 from pydantic import Field
+from shapely.geometry import Point
 from shapely.wkb import loads as wkb_loads
 from tqdm import tqdm
 
@@ -30,7 +53,7 @@ if TYPE_CHECKING:
 
 
 # ============================================================================
-# PRODUCT CONFIGURATION - Single Source of Truth
+# PRODUCT CONFIGURATION
 # ============================================================================
 #
 # To add a new product, just add ONE line here. Everything else is automatic.
@@ -192,7 +215,7 @@ PRODUCT_CONFIGS = [
 
 # Auto-generated schema from PRODUCT_CONFIGS (never edit this manually)
 PRODUCT_SCHEMA: dict[str, pl.DataType] = {
-    name: dtype
+    name: dtype()
     for name, _path, _reducer, _band, _coll_type, _unmask, dtype in PRODUCT_CONFIGS
 }
 
@@ -202,21 +225,22 @@ PRODUCT_SCHEMA: dict[str, pl.DataType] = {
 # ============================================================================
 
 
-def _import_earth_engine():
+def _import_earth_engine() -> Any:
     """Lazy import of Earth Engine with helpful error message."""
     try:
         import ee
 
-        return ee
     except ImportError as e:
         raise ImportError(
             "Google Earth Engine API is required for the GeoEnrich extension.\n"
             "Install with: pip install earthengine-api\n"
             "Then authenticate: earthengine authenticate"
         ) from e
+    else:
+        return ee
 
 
-def get_geoenrich_products() -> list[dict]:
+def get_geoenrich_products() -> list[dict[str, Any]]:
     """
     Get all available Earth Engine products for geospatial enrichment.
 
@@ -296,7 +320,7 @@ def morton_key(lon: float, lat: float, bits: int = 24) -> int:
     return (_part1by1(xi) << 1) | _part1by1(yi)
 
 
-def _chunks(seq: list, size: int):
+def _chunks(seq: list, size: int) -> Any:
     """Yield consecutive chunks from sequence of specified size."""
     for i in range(0, len(seq), size):
         yield seq[i : i + size]
@@ -385,27 +409,27 @@ class GeoEnrich(TortillaExtension):
 
     # Public configuration parameters
     variables: list[str] | None = Field(
-        None,
-        description="List of variable names to fetch. If None, fetches all available variables.",
+        default=None,
+        description="List of variable names to fetch. If None, fetches all 15 available variables. See PRODUCT_CONFIGS for complete list.",
     )
     scale_m: float = Field(
-        5120.0,
+        default=5120.0,
         ge=1.0,
-        description="Earth Engine reducer scale in meters. Smaller = higher resolution but slower.",
+        description="Earth Engine reducer spatial resolution in meters. Smaller values = higher resolution but slower queries and higher quota usage. Default 5120m (~5km).",
     )
     batch_size: int = Field(
-        250,
+        default=250,
         ge=1,
-        description="Number of points per Earth Engine reduceRegions call.",
+        description="Number of sample points per Earth Engine reduceRegions API call. Larger batches are faster but may hit EE memory limits. Default 250.",
     )
     max_concurrency: int = Field(
-        8,
+        default=8,
         ge=1,
-        description="Maximum number of concurrent Earth Engine requests.",
+        description="Maximum concurrent Earth Engine API requests via ThreadPoolExecutor. Higher values speed up processing but may hit EE rate limits. 8 is safe for most accounts.",
     )
     show_progress: bool = Field(
-        True,
-        description="Whether to display progress bar during processing.",
+        default=True,
+        description="Display tqdm progress bar during Earth Engine processing.",
     )
 
     def model_post_init(self, __context: Any) -> None:
@@ -423,6 +447,34 @@ class GeoEnrich(TortillaExtension):
         active_vars = self.variables or list(PRODUCT_SCHEMA.keys())
         return {f"geoenrich:{var}": PRODUCT_SCHEMA[var] for var in active_vars}
 
+    def get_field_descriptions(self) -> dict[str, str]:
+        """Return field descriptions for each field."""
+        all_descriptions = {
+            "geoenrich:elevation": "Mean elevation in meters (GLO-30 DEM)",
+            "geoenrich:cisi": "Mean of harmonized global critical infrastructure & index (CISI)",
+            "geoenrich:precipitation": "Mean annual precipitation in mm estimated from GPM data",
+            "geoenrich:temperature": "Mean annual temperature in °C estimated from MODIS LST data",
+            "geoenrich:soil_clay": "Clay content fraction (0-1) at surface layer, using the OpenLandMap dataset",
+            "geoenrich:soil_sand": "Sand content fraction (0-1) at surface layer, using the OpenLandMap dataset",
+            "geoenrich:soil_carbon": "Organic carbon content in g/kg at surface layer, using the OpenLandMap dataset",
+            "geoenrich:soil_bulk_density": "Bulk density in kg/m³ at surface layer, using the OpenLandMap dataset",
+            "geoenrich:soil_ph": "Soil pH (H2O) at surface layer, using the OpenLandMap dataset",
+            "geoenrich:gdp": "GDP per capita in USD (PPP 2022), Kummu et al. 2025",
+            "geoenrich:human_modification": "Human modification index v3 1990-2020, Theobald et al. 2024",
+            "geoenrich:population": "Population density from HRSL. Facebook High Resolution Settlement Layer",
+            "geoenrich:admin_countries": "Country name at centroid location",
+            "geoenrich:admin_states": "State/province name at centroid location",
+            "geoenrich:admin_districts": "District/county name at centroid location",
+        }
+
+        # Return only active variables
+        active_vars = self.variables or list(PRODUCT_SCHEMA.keys())
+        return {
+            key: all_descriptions[key]
+            for key in all_descriptions
+            if key.replace("geoenrich:", "") in active_vars
+        }
+
     def _get_active_products(self) -> list[dict]:
         """Get product configurations to process based on user selection."""
         all_products = get_geoenrich_products()
@@ -434,7 +486,8 @@ class GeoEnrich(TortillaExtension):
         """Extract coordinate points from DataFrame and sort spatially."""
         points = []
         for i, row in enumerate(df.iter_rows(named=True)):
-            geom = wkb_loads(row["stac:centroid"])
+            # Cast to Point to access .x and .y attributes safely
+            geom = cast(Point, wkb_loads(row["stac:centroid"]))
             points.append((i, float(geom.x), float(geom.y)))
 
         # Sort spatially using Morton key for better EE cache locality
@@ -463,7 +516,7 @@ class GeoEnrich(TortillaExtension):
         mode_cols = ["mode"] + [f"mode_{i}" for i in range(1, len(products))]
         rename_map = {
             mode_col: product["name"]
-            for mode_col, product in zip(mode_cols, products)
+            for mode_col, product in zip(mode_cols, products, strict=False)
             if mode_col in df.columns
         }
         return df.rename(rename_map)
@@ -522,6 +575,10 @@ class GeoEnrich(TortillaExtension):
         reducer_groups = self._group_products_by_reducer(products)
         chunks = list(_chunks(points, self.batch_size))
 
+        # Get expected schema for casting
+        product_names = [p["name"] for p in products]
+        expected_schema = {name: PRODUCT_SCHEMA[name] for name in product_names}
+
         with ThreadPoolExecutor(max_workers=self.max_concurrency) as executor:
             futures = [
                 executor.submit(self._reduce_chunk, chunk, reducer_groups, ee)
@@ -535,10 +592,20 @@ class GeoEnrich(TortillaExtension):
                 disable=not self.show_progress,
             ) as pbar:
                 for future in as_completed(futures):
-                    results.append(future.result())
+                    df_chunk = future.result()
+
+                    # Cast to expected schema BEFORE appending
+                    df_chunk = df_chunk.select(
+                        ["idx"]
+                        + [
+                            pl.col(name).cast(expected_schema[name], strict=False)
+                            for name in product_names
+                        ]
+                    )
+
+                    results.append(df_chunk)
                     pbar.update(1)
 
-        # Concatenate all batch results
         return pl.concat(results, how="vertical")
 
     def _compute(self, tortilla: "Tortilla") -> pl.DataFrame:
@@ -572,13 +639,8 @@ class GeoEnrich(TortillaExtension):
         # Sort by idx to match original order, then drop idx
         raw_results = raw_results.sort("idx").drop("idx")
 
-        # Add prefix and ensure schema (inline _finalize_schema)
+        # Add "geoenrich:" prefix
         product_names = [p["name"] for p in active_products]
         return raw_results.select(
-            [
-                pl.col(name)
-                .cast(PRODUCT_SCHEMA[name], strict=False)
-                .alias(f"geoenrich:{name}")
-                for name in product_names
-            ]
-        )
\ No newline at end of file
+            [pl.col(name).alias(f"geoenrich:{name}") for name in product_names]
+        )
